# frozen_string_literal: true

module Jive
  class Git
    attr_reader :shell

    def initialize(shell = ::Jive.shell)
      @shell = shell
    end

    def clone(slug, host: "github.com")
      dir = target_dir_for(slug, host: host)
      unless dir.exist?
        shell.execute([:mkdir, "-p", dir.parent.to_s])
        shell.execute([:git, "clone", "git@#{host}:#{slug}", dir])
      end
      cd(slug, host: host) if dir.exist?
    end

    def cd(slug, host: "github.com")
      dir = target_dir_for(slug, host: host)
      if dir.exist?
        shell.after_run([
          ["cd", dir],
          ["ctags", dir],
          ["setenv", "JIVE_LAST_RUN=#{Time.now.to_i}"]
        ])
      else
        clone(slug)
      end
    end

    def semantic_help
      <<~MESSAGE
        Format: <type>(<scope>): <subject>

        <scope> is optional

        feat: add hat wobble
        ^--^  ^------------^
        |     |
        |     +-> Summary in present tense.
        |
        +-------> Type: chore, docs, feat, fix, refactor, style, or test.

        chore: updating grunt tasks etc; no production code change
        docs: changes to the documentation
        feat: new feature for the user, not a new feature for build script
        fix: bug fix for the user, not a fix to a build script
        refactor: refactoring production code, eg. renaming a variable
        style: formatting, missing semi colons, etc; no production code change
        test: adding missing tests, refactoring tests; no production code change
      MESSAGE
    end

    private

    def target_dir_for(slug, host:)
      Pathname.new(Dir.home).join("src/#{host}/#{slug}")
    end
  end
end
